//	mtm.h
//

#ifndef _MTM_H
#define _MTM_H

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

// Typedefs used throughout, structure defined lowerd down
typedef enum { NOTE, PATCH, TEXT, TEMPO, PITCH, END, PARAM, BEND }EventType;
typedef struct _track Track, *Trackp;
typedef struct _list Eventlist, *Eventlistp;
typedef struct _event Event, *Eventp;
typedef struct _tlist Tlist, *Tlistp;
typedef struct _tsig Tsig, *Tsigp;
typedef struct _key Key, *Keyp;
typedef struct _bstyle Bstyle, *Bstylep;

// structure for each midi event
struct _event {
  Eventp next, prev;
  EventType type;		/* see definition above 		*/
  long start, end;		/* start and end time of event   	*/
  long remains;			/* start of remaining segment           */
  int pitch;			/* midi pitch 				*/
  union {
    int vel;			/* midi velocity			*/
    char *text;			/* for text pseudo events		*/
  };
  char oct, note, acc;		/* pitch converted 			*/
  char flags;			/* various flags		 	*/
};
//#define F_TIE	0x01		// note part of tied sequence
#define F_MOD	0x02		// note has been enharmonically changed
#define F_TIEF  0x04		// note is tied forwards
#define F_TIEB  0x08		// note is tied backwards

// structure to head a list of events
struct _list {	
  Eventp e;			/* pointer to first event in list 	*/
  Eventp p;			/* pointer to current/last event in list*/
  long count;			/* number of events in the list		*/
};

/* main structure holding information on each track */
struct _track {
  Eventlistp notes;	/* list of notes for this track */
  Eventlistp mevnts;	/* list of midi events for this track */
  int tied;
  int key;
  int hkey;
  short minp;		/* lowest pitch in track */
  short maxp;		/* highest pitch in track */
  short clef;		/* basic clef */
  short tr;
  short vel;
  short num;			/* useful to get back track number */
  char process;			/* contains actual notes */
  char *map1;			/* translation (with voice) */
  char *map2;			/* ditto (without voice)    */
  char name[128];		/* used in pnote	    */
  char acc[128];		/* ditto		    */
  char accp[128];		/* likewise		    */
} track;

/* Structure to hold time sigs as we read them in.	*/
struct _tsig {
  Tsigp next;
  long when;
  short num;
  short dnom;
};

/* structure to store list of text items */
struct _tlist {
  Tlistp next;
  char *text;
};

/* beamstyles from .midtomup file */
struct _bstyle {
  Bstylep next;
  int staff;			/* specific staff request */
  int bnum;
  int bdnom;
  char style[80];
};

/*
 * Hold key signatures offsets.
 * tick and new key info
 */
struct _key {
  Keyp next;
  long when;
  short nkey;
};

//	now define global variables
//
// track vector and current track pointers
extern Trackp *tracks;		/* main event tracks vector */
extern Trackp ct;		/* current track, and modifiers */
extern Tsigp tsigs;		/* list of Time Signature entries */
extern Tsigp tsigp;		/* pointer to last during input, work later */

extern Bstylep bstyles;		/* list of beamstyle entries 		*/
extern Keyp keysigs;		/* list of key signatures 		*/
extern Keyp keysigp;		/* pointer into list of key signatures 	*/

extern long ticks;		/* current position in piece 		*/
extern long bar_start;		/* start of current bar (in ticks)	*/
extern long bar_end;		/* end of current bar (in ticks)	*/
extern long piece_end;		/* end of piece, highest tick value	*/
extern long division;		/* resolution, #ticks/beat		*/
extern long startTime;		/* used to fudge pickups		*/
extern long Mf_currtime;	/* time as used in midifile		*/
extern long discards;		/* notes discarded (too short)		*/
extern int triplet;		/* Are we in a triplet? 		*/
extern int velfudge;		/* fudge to reduce number of vel inserts*/
extern int shortest;		/* length of shortest note we can resolve */
extern int maxtrk;		/* number of tracks to be allocated	*/
extern int ffactor;		/* default quantization 		*/
extern int trackno;		/* current track number			*/
extern int debug;		/* debugging output requested		*/
extern int warnFlag;		/* warning enabled			*/
extern int stretch;		/* stretch factor to round up lengths	*/
extern int shift;		/* slide notes forward by n ticks       */
extern int zflag;		/* -z option selected			*/
extern float pad;		/* -p pad factor, e.g. 3.5 add 3.5 beats of
				   rest for a pickup bar 		*/
extern char ifile[80];		/* input midifile name			*/
extern FILE *out;		/* output file				*/
extern FILE *inf;		/* input midifile			*/
extern long bytecount;		/* position in .midi file (debug)	*/
extern unsigned char debugbuff[32];	/* also for debugging		*/

/*
 * Keeps current measure of notes.  We'll use
 * this to avoid doing two accidentals on the
 * same note in the same measure.
 */
#define TABSIZE 32768
extern int accnotes[TABSIZE];
extern int accwhre;
/*
 * Similar to accnotes, but kept in seven note
 * format.  Instead of midi number, we keep note
 * name (as 1-7), octave, and accidental.
 */
extern int sccnotes[TABSIZE][3];
extern int sccwhre;

// finally, we define global functions
//
// the following are in input.c
extern void mtm_error(char *msg);	// display an error message
extern int  mtm_getc(void);		// read a byte from the midi file
extern void mtm_trackstart(void);	// process start of track entry
extern void mtm_arbitrary(int, char *);	// arbitrary meta-event
extern void mtm_trackend(void);		// process end of track entry
extern void mtm_header(int format, int ntrks, int ldivision);
extern void mtm_seqnum(int seq);	// process sequence number
extern void mtm_text(int type, int leng, char *mess);
extern void mtm_eot(void);
extern void mtm_tempo(long ltempo);
extern void mtm_smpte(int hr, int mn, int se, int fr, int ff);
extern void mtm_timesig(int nn, int dd, int cc, int bb);
extern void mtm_keysig(int sf, int mi);
extern void mtm_seqspecific(int type, int leng, char *mess);
extern void mtm_metamisc(int type, int leng, char *mess);
extern void mtm_sysex(int leng, char *mess);
extern void mtm_noteoff(int chan, int pitch, int vol);
extern void mtm_noteon(int chan, int pitch, int vol);
extern void mtm_pressure(int chan, int pitch, int press);
extern void mtm_parameter(int chan, int control, int value);
extern void mtm_pitchbend(int chan, int msb, int lsb);
extern void mtm_program(int chan, int program);
extern void mtm_chanpressure(int chan, int press);
extern void mtm_metaspecial(int type, int leng, char *mess);
extern FILE *efopen(char *name);

// the following are in mtm.c
extern void poctave(Trackp t, int nval);
extern int goctave(Trackp t, int nval);
extern void initTracks(void);
extern void addKey(long t, int k);
extern void addTsig(long t, int n, int d);

// following routines are in pnote.c
extern void pnote(int nval, Trackp to, int tied);
extern void resetaccs(Trackp t);
extern void setkeysig(Trackp t, int key);
extern void putNote(Trackp t, Eventp np);
extern void getPitch(Trackp t, Eventp np);
extern void loadNoteList(char *path);

#endif
